/*
 * This file is part of TONG.              http://www.nongnu.org/tong
 * Copyright 2004 Owen Swerkstrom
 *
 * TONG! is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TONG! is distributed in the hope that it will be useful and fun,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY of FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TONG!; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __PONG_H__
#define __PONG_H__

class Ball;
class Paddle;

#include <stdlib.h>
#include <math.h>

#include "SDL.h"
#include "SDL_image.h"

#include "tetris.h"
#include "option.h"
#include "media.h"

#define PI 3.14159265

#define BOUND_TOP    80
#define BOUND_LEFT   240
#define BOUND_RIGHT  400
#define BOUND_BOTTOM 400

#define NEWBALL_NOTICE 500  //half a second for a new ball to "appear"

#define OUT_OF_BOUNDS 0
#define MOVED         1
#define BOUNCED       2


class Ball {
 private:
  int ballstatus;
  int trappedbounces;
  float x, y;
  float angle; //0~360, but sin() takes radians so sin(angle*PI/180)
  float speed;    //pixels per second
  float startspeed;
  Uint32 lastmove, startmoving;
  bool visible;
  bool tetrisreacted;
  SDL_Surface *pongbits;
  SDL_Rect ballrect, destrect;
  void ReactWithStack(int, int, int, Bucket*, Option*);
  void ReactWithTetrad(int, int, int, Tetrad*, Tetrad*, Bucket*, Option*);
  void LoadGraphics();
 public:
  Ball();
  ~Ball();
  void Init(float);
  void Xmas();
  void UpdateTime();
  int Move(Paddle*, Bucket*, Tetrad*, Tetrad*, Option*);
  void Bounce(int direction);
  void Draw(SDL_Surface*);
};

class Paddle {
 private:
  int x, y;
  int xsize, ysize;
  float xspeed, yspeed;
  SDL_Surface *pongbits;
  SDL_Rect paddlerect, destrect;
  Uint32 lastdraw;
  void LoadGraphics();
 public:
  Paddle();
  ~Paddle();
  void Init(int, int);
  void Xmas();
  bool MouseMoved(int, int);
  bool MoveX(int);
  bool MoveY(int);
  float GetXSpeed();
  float GetYSpeed();
  bool IsAtX(int);
  bool IsAtY(int);
  void Draw(SDL_Surface*);
};

int pong2tetris(int, int);
int tetris2pongx(int);
int tetris2pongy(int);

#endif
